"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventEmitter = void 0;
const utils_1 = require("./utils");
const EventNames = {
    INITIALIZED: 'initialized',
    NEW_VARIABLES: 'newVariables',
    ERROR: 'error',
    VARIABLE_UPDATED: 'variableUpdated',
    FEATURE_UPDATED: 'featureUpdated',
};
class EventEmitter {
    constructor() {
        this.events = {};
    }
    subscribe(key, handler) {
        (0, utils_1.checkParamType)('key', key, 'string');
        (0, utils_1.checkParamType)('handler', handler, 'function');
        const eventNames = Object.keys(EventNames).map((e) => e.toLowerCase());
        if (!eventNames.includes(key) &&
            !key.startsWith(EventNames.VARIABLE_UPDATED) &&
            !key.startsWith(EventNames.FEATURE_UPDATED) &&
            !key.startsWith(EventNames.NEW_VARIABLES)) {
            throw new Error('Not a valid event to subscribe to');
        }
        else if (!this.events[key]) {
            this.events[key] = [handler];
        }
        else {
            this.events[key].push(handler);
        }
    }
    unsubscribe(key, handler) {
        (0, utils_1.checkParamType)('key', key, 'string');
        const eventNames = Object.keys(EventNames).map((e) => e.toLowerCase());
        if (!eventNames.includes(key)) {
            return;
        }
        else if (!handler) {
            this.events[key] = [];
        }
        else {
            this.events[key] = this.events[key].filter((eventHandler) => eventHandler !== handler);
        }
    }
    emit(key, ...args) {
        (0, utils_1.checkParamType)('key', key, 'string');
        const handlers = this.events[key];
        if (!handlers) {
            this.events[key] = [];
            return;
        }
        handlers.forEach((handler) => {
            handler(...args);
        });
    }
    emitInitialized(success) {
        this.emit(EventNames.INITIALIZED, success);
    }
    emitError(error) {
        this.emit(EventNames.ERROR, error);
    }
    emitVariableUpdates(oldVariableSet, newVariableSet, variableDefaultMap) {
        const keys = new Set(Object.keys(oldVariableSet).concat(Object.keys(newVariableSet)));
        let newVariables = false;
        keys.forEach((key) => {
            const oldVariableValue = oldVariableSet[key] && oldVariableSet[key].value;
            const newVariable = newVariableSet[key];
            const newVariableValue = newVariable && newVariableSet[key].value;
            if (JSON.stringify(oldVariableValue) !== JSON.stringify(newVariableValue)) {
                const variables = variableDefaultMap[key] && Object.values(variableDefaultMap[key]);
                if (variables) {
                    newVariables = true;
                    variables.forEach((variable) => {
                        var _a;
                        variable.value = newVariableValue !== null && newVariableValue !== void 0 ? newVariableValue : variable.defaultValue;
                        variable.isDefaulted = newVariableValue === undefined || newVariableValue === null;
                        (_a = variable.callback) === null || _a === void 0 ? void 0 : _a.call(variable, variable.value);
                    });
                }
                this.emit(`${EventNames.VARIABLE_UPDATED}:*`, key, newVariable);
                this.emit(`${EventNames.VARIABLE_UPDATED}:${key}`, key, newVariable);
            }
        });
        if (newVariables) {
            this.emit(`${EventNames.NEW_VARIABLES}`);
        }
    }
    emitFeatureUpdates(oldFeatureSet, newFeatureSet) {
        const keys = Object.keys(oldFeatureSet).concat(Object.keys(newFeatureSet));
        keys.forEach((key) => {
            const oldFeatureVariation = oldFeatureSet[key] && oldFeatureSet[key]._variation;
            const newFeature = newFeatureSet[key];
            const newFeatureVariation = newFeature && newFeatureSet[key]._variation;
            if (oldFeatureVariation !== newFeatureVariation) {
                this.emit(`${EventNames.FEATURE_UPDATED}:*`, key, newFeature);
                this.emit(`${EventNames.FEATURE_UPDATED}:${key}`, key, newFeature);
            }
        });
    }
}
exports.EventEmitter = EventEmitter;
//# sourceMappingURL=EventEmitter.js.map